//=============================================================================
//
//  CLASS AABBox
//
//=============================================================================
#ifndef RAYTRACER_AABBOX_H
#define RAYTRACER_AABBOX_H

//== INCLUDES =================================================================
#include <cfloat>
#include <QtOpenGL>
#include "Ray.h"


//== CLASS DEFINITION =========================================================

/*!
	Axis-aligned bounding box.
*/
class   AABBox
{

    //-------------------------------------------------------------- private data
private:
    VectorType3 m_v3Min;
    VectorType3 m_v3Max;

    //-------------------------------------------------------------- private methods
private:

public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    AABBox() :
    m_v3Min( FLT_MAX, FLT_MAX, FLT_MAX ),
    m_v3Max( -FLT_MAX, -FLT_MAX, -FLT_MAX )
    {
    };

    //! initializing constructor
    AABBox(
        const VectorType3   &v3Min, 
        const VectorType3   &v3Max )  :
    m_v3Min( v3Min ),
    m_v3Max( v3Max )
    {
    };

    //! destructor	
    ~AABBox()
    {
    };

    //----------------------------------------------------------- element access
    void setMin(
        const VectorType3   &v3Min )    
    {
        m_v3Min = v3Min;
    };

    
    const VectorType3 &getMin() const
    {
        return m_v3Min;
    };

    
    void setMax(
        const VectorType3   &v3Max )    
    {
        m_v3Max = v3Max;
    };

    
    const VectorType3 &getMax() const
    {
        return m_v3Max;
    };

    //----------------------------------------------------------- operators
    AABBox &operator+=(
        const VectorType3   &v3Offset ) 
    {
        m_v3Min += v3Offset;
        m_v3Max += v3Offset;
        return ( *this );
    };

    
    AABBox &operator*=(
        const VectorType3   &v3Scale )  
    {
        m_v3Min *= v3Scale;
        m_v3Max *= v3Scale;
        return ( *this );
    };

    
    AABBox &operator*=(
        const double  fScale )    
    {
        m_v3Min *= fScale;
        m_v3Max *= fScale;
        return ( *this );
    };

    //----------------------------------------------------------- functionality
public:
    
    const VectorType3 getCenter()
    {
        return ( m_v3Min + m_v3Max ) * 0.5;
    };

    
    const VectorType3 getSize()
    {
        return ( m_v3Max - m_v3Min );
    };

    
    void Reset()
    {
        m_v3Min[0] = m_v3Min[1] = m_v3Min[2] = FLT_MAX;
        m_v3Max[0] = m_v3Max[1] = m_v3Max[2] = -FLT_MAX;
    };

    //!Extend by point.
    void    extendBy( const VectorType3 &v3Point );

    //!Extend by box.
    void    extendBy( const AABBox &clBox );

    //!	intersect?
    bool    intersect( const Ray &clRay ) const;

	//Render in OpenGL
	void	renderGL();
};

//=============================================================================
#endif // RAYTRACER_AABBOX_H

